@@ -27,9 +27,10 @@ else if (file_exists('./plugins/compatibility/functions.php'))
   * hostname portion of the current URL), a per user file, etc.
   *
   */
-function overrideSmConfig() {
+function overrideSmConfig() 
+{
 
-   global $allVirtualDomainsAreUnderOneSSLHost, 
+   global $allVirtualDomainsAreUnderOneSSLHost, $PHP_SELF,
           $smHostIsDomainThatUserLoggedInWith, $virtualDomainDataDir, 
           $data_dir, $domain, $virtualDomains, $plugins,  
           $squirrelmail_plugin_hooks, $useSessionBased;
@@ -65,10 +66,21 @@ function overrideSmConfig() {
   if ($useSessionBased)
   {
      global $config_override;
-     compatibility_sqsession_unregister('config_override');
-     // If we don't initialize this, it will work 
-     // together with the multilogin plugin
-     // $config_override = array();
+
+
+     // only clean up when first logging in
+     //
+     if (strpos($PHP_SELF, 'redirect.php') !== FALSE
+      || strpos($PHP_SELF, 'login.php') !== FALSE)
+     {
+        compatibility_sqsession_unregister('config_override');
+
+        // If we don't initialize this, it will work
+        // together with the multilogin plugin
+        // $config_override = array();
+     }
+     else
+        compatibility_sqextractGlobalVar('config_override');
   }
 
 
@@ -140,10 +152,15 @@ function overrideSmConfig() {
 //       the session_recall patch, however it will also unfortunately
 //       mask errors in the config file.  removing this line could
 //       possibly help debug non-functional vlogin installations
+  //
+  // only need to do this stuff when first logging in
+  //
+  if (strpos($PHP_SELF, 'redirect.php') !== FALSE
+   || strpos($PHP_SELF, 'login.php') !== FALSE)
   if (is_array($virtualDomains))
   foreach (array_keys($virtualDomains) as $virtualDomain) {
 
-    if (stristr($hostname, $virtualDomain) || $virtualDomain == '*') {
+    if (stristr($hostname, (string)($virtualDomain)) || $virtualDomain == '*') {
 
 
       // limit usage of global domain to first entry only
@@ -174,7 +191,7 @@ function overrideSmConfig() {
            if (strpos($value, '###USERNAME###') !== FALSE && !empty($user))
               $value = str_replace('###USERNAME###', 
                                $user, 
-                               $the_data_dir);
+                               $value);
         }
          
         // Go ahead and replace the string ###USERNAME###
@@ -246,6 +263,7 @@ function overrideSmConfig() {
            {
               global $$setting;
               eval('$$setting = ' . $value . ';');
+              $config_override[$setting] = $value;
            }
            else
               $config_override[$setting] = $value;
@@ -319,6 +337,122 @@ function overrideSmConfig() {
 
 
 
+/** 
+  * Figures out just which hostname is the right one to add to the end 
+  * of the username.  This function does *NOT* care whether or not it
+  * will in fact be added to the username; it is only responsible for
+  * getting the right hostname and no more.
+  *
+  * @return string The expected host name
+  *
+  */
+function determine_user_hostname()
+{
+
+   global $useDomainFromVirtDomainsArray, $virtualDomains, $securePort,
+          $allVirtualDomainsAreUnderOneSSLHost, $forceLowercase;
+
+
+   // get vlogin config settings
+   //
+   include_once (SM_PATH . 'plugins/vlogin/data/config.php');
+
+
+   // get global variable for versions of PHP < 4.1
+   //
+   if (!compatibility_check_php_version(4,1)) {
+      global $HTTP_SERVER_VARS;
+      $_SERVER = $HTTP_SERVER_VARS;
+   }
+
+
+   // grab hostname into local var
+   //
+   if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+      $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
+   else
+      $hostname = $_SERVER['HTTP_HOST'];
+
+
+
+   // grab server port 
+   //
+   if (isset($_SERVER['SERVER_PORT']))
+      $serverPort = $_SERVER['SERVER_PORT'];
+   else
+      $serverPort = 0;
+
+
+
+   // what's this server's HTTPS port?
+   //
+   if (isset($securePort))
+      $httpsPort = $securePort;
+   else
+      $httpsPort = '443';
+
+
+
+  // for sites where virtual host is pegged on the end of the main
+  // site's URL (usually for single-certificate SSL hosting), get 
+  // the actual host name out of the PHP_SELF portion of the URL
+  //
+// old way:
+//  if (isset($_SERVER['HTTPS']) 
+  if ($serverPort == $httpsPort
+   && $allVirtualDomainsAreUnderOneSSLHost) 
+  {
+
+     preg_match('/[\/]*(.*?)(\/|$)/', $_SERVER['PHP_SELF'], $matches);
+     $hostname = $matches[1];
+
+  }
+
+
+
+  // go lowercase if needed
+  //
+  if ($forceLowercase)
+     $hostname = strtolower($hostname);
+
+
+
+   // allow the use of the domain setting from the $virtualDomains
+   // array, which is a heck of a lot simpler...
+   //
+   if ($useDomainFromVirtDomainsArray)
+   {
+
+      if (is_array($virtualDomains))
+      foreach (array_keys($virtualDomains) as $virtualDomain)
+      {
+
+         if (stristr($hostname, (string)($virtualDomain)) || $virtualDomain == '*')
+         {
+
+            if (isset($virtualDomains[$virtualDomain]['domain']))
+               $hostname = $virtualDomains[$virtualDomain]['domain'];
+
+
+            // exit this loop, unless this is the global default
+            //
+            if( $virtualDomain != '*' ) break;
+
+         }
+
+      }
+
+   }
+   else
+      $hostname = deconstructDomainName($hostname);
+
+
+   return $hostname;
+
+}
+
+
+
 /** 
   * Remap Username When Logging In
   * 
@@ -335,8 +469,9 @@ function vlogin_domain_do()
   global $plugins, $login_username, $$login_username, $debug, $data_dir, 
          $foundLoginAlias, $at, $dot, $dontUseHostName, $atConversion,
          $sendmailVirtualUserTable, $putHostNameOnFrontOfUsername,
-         $allVirtualDomainsAreUnderOneSSLHost, $prefs_dsn, $useSessionBased,
-         $virtualDomains, $removeDomainIfGiven, $alwaysAddHostName;
+         $prefs_dsn, $useSessionBased, $removeDomainIfGiven, 
+         $alwaysAddHostName, $usernameReplacements, $forceLowercase,
+         $prefs_are_cached, $prefs_cache;
 
 
   // figure out where prefs are stored
@@ -347,45 +482,23 @@ function vlogin_domain_do()
       $prefsInDB = false;
 
 
-  // get global variable for versions of PHP < 4.1
-  //
-  if (!compatibility_check_php_version(4,1)) {
-    global $HTTP_SERVER_VARS;
-    $_SERVER = $HTTP_SERVER_VARS;
-  }
-
-
-  // grab hostname into local var
-  //
-  if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
-     $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
-  else
-     $hostname = $_SERVER['HTTP_HOST'];
-
-
-  // for sites where virtual host is pegged on the end of the main
-  // site's URL (usually for single-certificate SSL hosting), get 
-  // the actual host name out of the PHP_SELF portion of the URL
-  //
-  if ($allVirtualDomainsAreUnderOneSSLHost) {
-
-     preg_match('/[\/]*(.*?)(\/|$)/', $_SERVER['PHP_SELF'], $matches);
-     $hostname = $matches[1];
-
-  }
-
-
   // get vlogin config settings
   //
   include_once (SM_PATH . 'plugins/vlogin/data/config.php');
 
 
 
+  if ($useSessionBased)
+     overrideSmConfig();
+
+
+
   // what username did the user submit to us?
   //
   $user = $login_username;
   
 
+
   // if password_forget is loaded, use the obfuscated name
   //
   if (in_array('password_forget', $plugins)) {
@@ -395,6 +508,7 @@ function vlogin_domain_do()
   }
 
 
+
   // if user logged in with full email address, 
   // chop off domain info if needed
   //
@@ -404,6 +518,11 @@ function vlogin_domain_do()
   }
 
 
+
+  $hostname = determine_user_hostname();
+
+
+
   // check for login_alias plugin
   //
   if (in_array('login_alias',$plugins)) 
@@ -419,28 +538,40 @@ function vlogin_domain_do()
     else 
     {
 
+      // make sure prefs file exists
+      //
+      if (!$prefsInDB)
+         touch(getHashedFile('login_alias', $data_dir, "login_alias.pref"));
+
+
       // check for login alias here and return if found
       // (but only if it is in the domain being used
       // to log in when dontUseHostName is off)
       //
-      $filename = $data_dir.'login_alias.pref';
-      if ((!$prefsInDB && file_exists($filename)) || $prefsInDB) 
+      $prefs_are_cached = false;
+      $prefs_cache = array();
+      session_unregister('prefs_are_cached');
+      session_unregister('prefs_cache');
+
+      $loginAlias = getPref($data_dir, 'login_alias', $user);
+
+      $prefs_are_cached = false;
+      $prefs_cache = array();
+      session_unregister('prefs_are_cached');
+      session_unregister('prefs_cache');
+
+
+      if (!empty($loginAlias))
       {
-        $loginAlias=getPref($data_dir,'login_alias',$user);
-
-        if (!empty($loginAlias))
-        {
-          if (!$dontUseHostName)
-          {
-            if (strpos($loginAlias, deconstructDomainName($hostname)) !== FALSE)
-              return;
-          }
-          else
-          {
+         if (!$dontUseHostName)
+         {
+            if (strpos($loginAlias, $hostname) !== FALSE)
+               return;
+         }
+         else
+         {
             return;
-          }
-
-        }
+         }
 
       }
 
@@ -457,59 +588,18 @@ function vlogin_domain_do()
 
 
 
-/* what was I thinking... 
-
-  // for those sysadmins who like to torture their 
-  // users and require a domain as part of the login...
+  // go lowercase if needed
   //
-  if( $forceDomainInUsername )
-  {
-
-    if ( strpos($user, $at) === FALSE 
-      || substr($user, strpos($user, $at) + strlen($at)) != deconstructDomainName($hostname))
-    {
-      logout_error(_("Username must include domain name") . ' ' . deconstructDomainName($hostname));
-      exit;
-    }
-
-  }
+  if ($forceLowercase)
+     $user = strtolower($user);
 
 
 
-  // rare cases where we want to strip off domain from
-  // username after everything has been done above
+  // replace any characters as necessary
   //
-  if ( $removeDomainAfterDomainCheck && strpos($user, $at) !== FALSE ) 
-  {
-     $user = substr($user, 0, strpos($user, $at));
-  }
-
-   $forceDomainInUsername
-   ----------------------
-   can be used to require the user to input the domain portion of
-   their username (which MUST match the hostname as reconstructed
-   using the settings you have made here).  This is not the best
-   way to require that the user's domain matches the URL and is a 
-   big hassle for users most of the time, so it is not a recommended 
-   option.  If you are looking for a way to ensure the domain on the 
-   username matches the URL, use $removeDomainIfGiven and make sure 
-   $dontUseHostName is turned off.  This option is mostly useful if 
-   you have a combination of usernames both WITH and WITHOUT (in 
-   combination with $removeDomainAfterDomainCheck) hostname portions 
-   and are thus using $dontUseHostName, but you still want to make 
-   sure the username's domain portion matches the URL.
-
-   $removeDomainAfterDomainCheck
-   -----------------------------
-   will remove the domain portion of the username if given.  This 
-   option should not be used in favor of $removeDomainIfGiven if at
-   all possible.  This setting is usually only used when 
-   $forceDomainInUsername is turned on so usernames that shouldn't
-   have domain portions on them...............
-
-DUH.  Took me this long to figure out that I was spinnin my wheels for
-a user who was asking the impossible.  This just here for posterity...
-*/
+  if (is_array($usernameReplacements))
+    foreach ($usernameReplacements as $chars => $repl)
+      $user = str_replace($chars, $repl, $user);
 
 
 
@@ -519,9 +609,6 @@ a user who was asking the impossible.  This just here for posterity...
   if( $alwaysAddHostName || (!$dontUseHostName && strpos($user, $at) === FALSE) ) {
 
 
-    $hostname = deconstructDomainName($hostname);
-
-
     // assign realname using parsed hostname
     //
     if ($putHostNameOnFrontOfUsername)
@@ -543,10 +630,6 @@ a user who was asking the impossible.  This just here for posterity...
   }
 
 
-  if ($useSessionBased)
-     overrideSmConfig();
-
-
   // override settings on a per-user basis
   //
   perUserOverride($realname);
@@ -572,7 +655,8 @@ a user who was asking the impossible.  This just here for posterity...
      echo '</h4><br><hr>';
      // was confusing to people using $dontUseHostName 
      if (!$dontUseHostName) echo '<br>$hostname is ' . $hostname . '<br>';
-     echo '<br>PHP_SELF is ' . $_SERVER['PHP_SELF'] . '<br>';
+     echo '<br>PHP_SELF is ' . (isset($_SERVER['PHP_SELF']) ? $_SERVER['PHP_SELF'] : 'not globally accessible')
+        . '<br>';
 global        $imapServerAddress;
 echo '<hr>' . $imapServerAddress . '<hr>';
      echo '<br><hr></body></html>';
@@ -1043,6 +1127,8 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
    global $at;
 
    $catchallLogin = '';
+   $remappedUsername = '';
+   $atPos = strpos($user, $at);
 
 
    if ($VIRTTABLE = @fopen ($sendmailVirtualUserTable, 'r'))
@@ -1051,8 +1137,8 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
       // be ready to use catchall address - prepare domain name
       //
       $domainName = '';
-      if (strpos($user, $at) !== FALSE)
-         $domainName = substr($user, strpos($user, $at));
+      if ($atPos !== FALSE)
+         $domainName = substr($user, $atPos);
 
 
       while (!feof($VIRTTABLE))
@@ -1079,12 +1165,13 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
             $catchallLogin = $matches[2];
 
 
-         // if user is found, get remapped login and return
+         // if user is found, get remapped login and finish looping
          //
          if ($user === $matches[1]) 
          {
+            $remappedUsername = $matches[2];
             fclose($VIRTTABLE);
-            return $matches[2];
+            break;
          }
 
       }
@@ -1096,23 +1183,38 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
    }
 
 
-   // if we got here and a catchall login is available, use it
+   // if we get here without having found a direct match
+   // and a catchall login is available, use it
    //
-   if (!empty($catchallLogin))
+   if (empty($remappedUsername) && !empty($catchallLogin))
    {
 
       // remove extraneous stuff we don't need off end (begins with plus sign)
       //
       $catchallLogin = preg_replace('/\+%\S+\s*$/', '', $catchallLogin);
 
-      return $catchallLogin;
+      $remappedUsername = $catchallLogin;
 
    }
 
 
-   // otherwise, unaltered username is returned
+   // otherwise, use unaltered username
    //
-   return $user;
+   else
+      $remappedUsername = $user;
+
+
+
+   // do variable replacement if needed
+   //
+   if ($atPos !== FALSE && strpos($remappedUsername, '%1') !== FALSE)
+   {
+      $usernameOnly = substr($user, 0, $atPos);
+      $remappedUsername = str_replace('%1', $usernameOnly, $remappedUsername);
+   }
+
+
+   return $remappedUsername;
 
 }
 
@@ -1141,13 +1243,45 @@ function vlogin_display_options_do()
       if (!$useSessionBased)
       {
 
+         // get global variable for versions of PHP < 4.1 
+         //
+         if (!compatibility_check_php_version(4,1)) 
+         {
+            global $HTTP_SERVER_VARS;
+            $_SERVER = $HTTP_SERVER_VARS;
+         }
+
+
+         // grab hostname into local var
+         //
+         if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+            $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
+         else
+            $hostname = $_SERVER['HTTP_HOST'];
+         $hostname_stripped = deconstructDomainName($hostname);
+
+
          // allow inclusion of external configuration files for each domain
          //
          if (file_exists(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php'))
             include_once(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php');
 
+
+         // get virtual domain config items
+         //
+         $overrides = array();
          if (is_array($virtualDomains))
-            $overrides = $virtualDomains;
+         foreach (array_keys($virtualDomains) as $virtualDomain) 
+         {
+
+            if (stristr($hostname, (string)($virtualDomain)) || $virtualDomain == '*') 
+            {
+
+               $overrides = $virtualDomains[$virtualDomain];
+
+            }
+
+         }
 
       }
 
